'    ***************************************************************************
'    *                         Picbasic Web Server II                          *
'    ***************************************************************************
'    * par David REY le 14/08/2005               http://www.RDElectronique.com *
'    ***************************************************************************
'    * Exemple de structure GET : GET /k.html                                  *
'    * k est ne n de la page html demande compris entre 1 et 3               *
'    * 1.html : tat des sorties logiques des PCF8574                          *
'    *          variable = 'In'                                                *
'    *          exemple de requte GET de mise  jour des sorties              *
'    *          GET /1.html?PXn=xxx&PXn+1=xxx&...                              *
'    * 2.html : tat des entres logiques des PCF8574A                         *
'    *          variable = 'On'                                                *
'    * 3.html : tensions analogiques lues sur les broches du PicBasic          *
'    *          variables = '$A0','$A1','$A2','$A3'                            *
'    * Les 3 pages html sont stockes dans la mmoire eeprom I2C               *
'    * $CP = compteur de connexions depuis dernier RESET                       *
'    ***************************************************************************


     '**************************************************************************
     'DECLARATION DES CONSTANTES
     '**************************************************************************

     const BYTE k_array=(0,'A','O','I','C')                    'Liste des variables

     Const BDS = 50                                            'Vitesse de transmission (103=9600bds/50=19200bds)
     Const RXD = 8                                             'Broche utilise pour rception srie=I/O8
     Const TXD = 9                                             'Broche utilise pour mission srie=I/O9
                                                     
     Const SDA    = 10                                         'Broche donne I2C=I/O10
     Const SCL    = 11                                         'Broche horloge I2C=I/O11
     Const CTRL_24LC_R = &b10100001                            'Octet de contrle lecture mmoire I2C
     Const CTRL_24LC_W = &b10100000                            'Octet de contrle ecriture mmoire I2C

     Const Nb_PCF8574 = 0                                      'Nombre de PCF8574 sur le bus I2C +1
     Const CTRL_PCF8574_R = &B01000001                         'Commande de lecture
     Const CTRL_PCF8574_W = &B01000000                         'Commande d'criture

     Const Nb_PCF8574A = 0                                     'Nombre de PCF8574A sur le bus I2C +1
     Const CTRL_PCF8574A_R = &B01110001                        'Commande de lecture
     Const CTRL_PCF8574A_W = &B01110000                        'Commande d'criture


    '***************************************************************************
    'DECLARATION DES VARIABLES
    '***************************************************************************
     DIM Adr   AS INTEGER                                      'Pointeur Adressage eeprom I2C
     DIM V     AS INTEGER                                      'Divers
     DIM p     AS BYTE                                         'Divers
     DIM i     AS BYTE                                         'Divers
     DIM j     AS BYTE                                         'Divers
     DIM k     AS BYTE                                         'Divers
     DIM l     AS BYTE                                         'Divers
     DIM Cpt   AS INTEGER                                      'Compteur de connexion

     DIM GET(50) AS BYTE                                       'Stockage requte HTTP


    '***************************************************************************
    'PROGRAMME PRINCIPAL
    '***************************************************************************

    '===========================================================================
    'INITIALISATIONS
    '===========================================================================

     Cpt=0                                                     'Initialisation du compteur de connexions

INIT:

     Adr=0                                                     'Initialisation pointeur adresse eeprom I2C

    'Initialisation de l'afficheur LCD -----------------------------------------
     SET PICBUS HIGH                                           'PICBUS  19200bds
     LCDINIT                                                   'Initialisation LCD
     CLS                                                       'Efface LCD
     LOCATE 0,0 : PRINT "PB-3B WEB SERVER"                     'Message d'accueil Ligne 1
     LOCATE 0,1 : PRINT ""                                     'Message d'accueil Ligne 2


    '===========================================================================
    'MISE A JOUR DES SORTIES DE LA CARTE ET DE L'ECRAN LCD
    '===========================================================================

DEBUT:

     'Attente connexion client http --------------------------------------------
     SERIN RXD,BDS,0,1000,DEBUT,[UNTIL(13),GET(0)~49]           'Enregistrement des caractres reus dans la variable GET
                                                                'tant que caractre <> retour chariot (=13)

     'Redirection vers sous programme de programmation eeprom ------------------
     IF GET(0)="P" THEN                                         'Si 3 premiers caractres = "PRG"
       IF GET(1)="R" AND GET(2)="G" THEN GOTO PRG               'enregistre les caractres suivants dans l'eeprom (page HTML)
     END IF                                                     '

     'Incrmentation du compteur de connexions ---------------------------------
     Cpt=Cpt+1

     SEROUT TXD,BDS,0,0,["HTTP/1.0 200 OK",13,10]               'Accus de rception requte accomplie correctement

     'Aiguillage en fonction de la page demande -------------------------------
     k=GET(5)                                                   'GET /k.html?PX=xxx
     k=k-48                                                     'ASCII->DEC k=n de la page html demande
     IF k>3 THEN k=0

     j=0                                                        'RAZ j
     FOR i=0 TO 49                                              'Lecture variable tableau GET
       IF GET(i)="?" THEN j=i+4                                 'Si "?" dtect => mmorisation dans j position dbut variables passes en GET
     NEXT i                                                     '

     ON k GOTO 9,1,2,3                                          'branche sur un n de ligne en fonction de la valeur de k=n de la page html demande

    '---------------------------------------------------------------------------
    'PAGE 1.html : SORTIES LOGIQUES DES PCF8574
    '---------------------------------------------------------------------------
1    Adr=0                                                      'dbut de la page dans eeprom srie
     IF j=0 THEN GOTO 9                                         'si pas de variable passe en GET branche sur ligne n9
     FOR p=0 TO Nb_PCF8574                                      'balayage des PCF8574P connects sur le bus I2C
       GOSUB ASCII2DEC                                          'conversion nombre ASCII -> nombre DECIMAL
       l=CTRL_PCF8574_W                                         '
       GOSUB I2C_W_PP                                           'SP ECRITURE i sur PCF8574AP
       j=j+5                                                    'se positionne sur prochaine variable
      NEXT p                                                    '
      GOTO 9                                                    'branche sur ligne n9

    '----------------------------------------------------------------------------
    'PAGE 2.html : ENTREES LOGIQUES DES PCF8574A
    '----------------------------------------------------------------------------
2    GOSUB TAILLE_PAGE                                          'dbut de la page dans eeprom srie
     GOTO 9                                                     'branche sur ligne n9

    '----------------------------------------------------------------------------
    'PAGE 3.html : ENTREES ANALOGIQUES DU PICBASIC
    '----------------------------------------------------------------------------
3    GOSUB TAILLE_PAGE                                          'dbut de la page dans eeprom srie
     GOTO 9                                                     'branche sur ligne n9


    '===========================================================================
    'ENVOI DE LA PAGE HTML AU CLIENT HTTP
    '===========================================================================

    'Envoi de l'entte http ----------------------------------------------------
9   SEROUT TXD,BDS,0,0,["Content-Type: text/html",13,10]       'Type de contenu : html=text/html (si wap=text/vnd.wap.wml)
    SEROUT TXD,BDS,0,0,["Content-Length: 8192",13,10]          'Longueur du corps de la rponse
    SEROUT TXD,BDS,0,0,[13,10]                                 'Ligne vierge

    'Envoi du code html --------------------------------------------------------

SUITE_RAZ:

    GOSUB I2C_START                                            'Condition de START
    SHIFTOUT SCL,SDA,2,CTRL_24LC_W,8                           'Octet de controle ECRITURE
    i=Adr/256                                                  'Calcul MSB
    SHIFTOUT SCL,SDA,2,i,8                                     'Adresse du 1er octet  lire (MSByte)
    i=Adr                                                      'Calcul LSB
    SHIFTOUT SCL,SDA,2,i,8                                     'Adresse du 1er octet  lire (LSByte)
    GOSUB I2C_START                                            'Condition de START
    SHIFTOUT SCL,SDA,2,CTRL_24LC_R,8                           'Octet de controle LECTURE

    i=0:j=0:k=0

SUITE:

    j=SHIFTIN(SCL,SDA,1,8)                                     'Lecture de la donne
    Adr=Adr+1                                                  '
    IF j=0 THEN                                                '0 = NULL = sparateur de fichier HTML
      SEROUT TXD,BDS,0,0,["<P><CENTER><A HREF='http://www.rdelectronique.com' target=_blank>www.RDElectronique.com</A></P>"]
      GOSUB I2C_NOACK                                          'No Acknowledge
      GOSUB I2C_STOP                                           'Condition de STOP
      GOTO DEBUT                                               'Branche sur l'tiquette DEBUT
    ELSE                                                       '
      GOSUB I2C_ACK                                            'Envoi acknowledge
    END IF                                                     '

    ON k GOTO 1000,10,20,30                                    'branche sur une ligne en fonction de la valeur de k (ex: si k=1 branche sur la ligne 1)

    '---------------------------------------------------------------------------
    'LECTURE DES ENTREES ANALOGIQUE 'A'
    '---------------------------------------------------------------------------
10  GOSUB I2C_NOACK                                            'Termine le dialogue avec la mmoire I2C
    GOSUB I2C_STOP                                             'Termine le dialogue avec la mmoire I2C
    j=j-48                                                     'ASCII -> DEC
    V=ADIN(j)                                                  'Lecture donne analogique
    SEROUT TXD,BDS,0,0,[DEC(V)]                                'Envoi valeur mesure
    GOTO SUITE_RAZ                                             'Branche sur l'tiquette SUITE_RAZ

    '---------------------------------------------------------------------------
    'LECTURE NUMERIQUE DES SORTIES 'O'
    '---------------------------------------------------------------------------
20  GOSUB I2C_NOACK                                            'Termine le dialogue avec la mmoire I2
    GOSUB I2C_STOP                                             'Termine le dialogue avec la mmoire I2
    FOR k=0 TO Nb_PCF8574                                      'Parcours tous les PCF8574
      l=CTRL_PCF8574_R                                         '
      GOSUB I2C_R_PP                                           'Lecture des sorties
    NEXT k                                                     'PCF8574 suivant
    GOTO SUITE_RAZ                                             'Branche sur l'tiquette SUITE_RAZ

    '---------------------------------------------------------------------------
    'LECTURE NUMERIQUE DES ENTREES 'I'
    '---------------------------------------------------------------------------
30  GOSUB I2C_NOACK                                            'Termine le dialogue avec la mmoire I2
    GOSUB I2C_STOP                                             'Termine le dialogue avec la mmoire I2
    FOR k=0 TO Nb_PCF8574A                                     'Parcours tous les PCF8574
      l=CTRL_PCF8574A_R
      GOSUB I2C_R_PP                                           'Lecture des entres
    NEXT k                                                     'PCF8574 suivant
    GOTO SUITE_RAZ                                             'Branche sur l'tiquette SUITE_RAZ

    '---------------------------------------------------------------------------
    'COMPTEUR DE CONNEXIONS
    '---------------------------------------------------------------------------
50  IF j="P" THEN SEROUT TXD,BDS,0,0,[DEC(Cpt)]                'Envoi donne compteur
    GOTO SUITE_RAZ                                             'Branche sur l'tiquette SUITE_RAZ


   '----------------------------------------------------------------------------

1000
    IF i="$" THEN                                              'si caractre n-1 = $
     FOR k=1 TO 10                                             '
      IF k_array(k)=j THEN GOTO SUITE                          'Recherche dans k_array la position
     NEXT k                                                    '
    END IF                                                     '
    IF j<>"$" THEN SEROUT TXD,BDS,0,0,[j]                      'Envoi resultat sur port srie
    i=j                                                        'Caractre n-1 = caractre n
    GOTO SUITE                                                 'Branche sur tiquette SUITE => Lecture donne suivante


    '===========================================================================
    'PROGRAMMATION DE L'EEPROM (fichier HTML envoy sur la SCI)
    '===========================================================================

PRG:

    LOCATE 0,0 : PRINT "HTML -> EEPROM  "                      'Affichage commentaire LCD

    'Programmation de l'eeprom srie -------------------------------------------
    Adr=0:V=4095:i=255                                         'Initialisation pointeur adresse eeprom
PAT:SERIN RXD,BDS,0,1000,PAT,[j]                               'Lecture d'un caractre sur SCI
    IF i=0 THEN                                                'Si i=0
      k=Adr/256                                                'MSB pointeur adresse
      EEWRITE V,k                                              'Enregistrement dans eeprom du Picbasic
      V=V-1                                                    '
      k=Adr                                                    'LSB pointeur adresse
      EEWRITE V,k                                              'Enregistrement dans eeprom du Picbasic
      V=V-1                                                    '
    END IF                                                     '
    GOSUB I2C_WRITE_BYTE                                       'Ecriture du caractre dans l'eeprom
    LOCATE 0,1 : PRINT "Octets : ",DEC(Adr)                    'Affichage sur l'cran LCD le nombre d'octets programms
    SEROUT TXD,BDS,0,0,[j]                                     'Renvoi du caractre sur SCI pour confirmation
    i=j                                                        '
    IF j<>26 THEN GOTO PAT                                     '26 signale la fin du transfert des fichiers HTML
    DELAY 2500                                                 'Tempo 2,5s
    GOTO INIT                                                  'Branche sur l'tiquette INIT




    '***************************************************************************
    ' SOUS-PROGRAMMES
    '***************************************************************************

    '===========================================================================
    ' ENREGISTRE LA POSITION DE DEBUT DE LA PAGE HTML
    '===========================================================================
TAILLE_PAGE:                                                  '
    k=k-2                                                     '
    k=k*2                                                     '
    V=4095-k                                                  '
    i=EEREAD(V)                                               '
    V=V-1                                                     '
    l=EEREAD(V)                                               '
    Adr=i*256                                                 '
    Adr=Adr+l                                                 '
    RETURN                                                    '

    '===========================================================================
    ' MEMOIRE EEPROM I2C : ECRITURE 1 octet
    '===========================================================================
I2C_WRITE_BYTE:
    GOSUB I2C_START                                           'Condition de START
    SHIFTOUT SCL,SDA,2,CTRL_24LC_W,8                          'Octet de contrle LECTURE
    i=Adr/256                                                 'Octet de poids fort
    SHIFTOUT SCL,SDA,2,i,8                                    'Adresse dbut criture (MSByte)
    i=Adr                                                     'Octet de poids faible
    SHIFTOUT SCL,SDA,2,i,8                                    'Adresse dbut criture (LSByte)
    SHIFTOUT SCL,SDA,2,j,8                                    'Ecriture en eeprom de j
    Adr=Adr+1                                                 'Incrementation pointeur adresse
    DELAY 5                                                   'Tempo 5ms
    GOSUB I2C_STOP                                            'Condition de STOP
    DELAY 5                                                   'Tempo 5ms
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'SORTIES PARALLELES I2C : ECRITURE 1 octet
    '===========================================================================
I2C_W_PP:
    k=p<<1                                                    'Rotation  gauche
    k=k OR l                                                  '
    GOSUB I2C_START                                           'Condition de START
    SHIFTOUT SCL,SDA,2,k,8                                    'Octet de controle ECRITURE
    SHIFTOUT SCL,SDA,2,i,8                                    'Adresse du 1er octet  lire (MSByte)
    GOSUB I2C_STOP                                            'Condition de STOP
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'ENTREES PARALLELES I2C : LECTURE 1 octet
    '===========================================================================
I2C_R_PP:
    j=k<<1                                                    'Rotation  gauche de l'adresse
    j=j OR l                                                  'Constitution de l'adresse
    GOSUB I2C_START                                           'Condition de START
    SHIFTOUT SCL,SDA,2,j,8                                    'Octet de controle LECTURE
    l=SHIFTIN(SCL,SDA,1,8)                                    'Lecture
    SEROUT TXD,BDS,0,0,[DEC(l),";"]                           'Evoi valeur
    GOSUB I2C_NOACK                                           'NoAcknowledge
    GOSUB I2C_STOP                                            'Stop
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'I2C : START
    '===========================================================================
I2C_START:
    OUT SCL,0                                                 'SCL=0
    OUT SDA,1                                                 'SDA=1
    OUT SCL,1                                                 'SCL=1
    OUT SDA,0                                                 'SDA=0
    OUT SCL,0                                                 'SCL=0
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'I2C : STOP
    '===========================================================================
I2C_STOP:
    OUT SCL,0                                                 'SCL=0
    OUT SDA,0                                                 'SDA=0
    OUT SCL,1                                                 'SCL=1
    OUT SDA,1                                                 'SDA=1
    OUT SCL,0                                                 'SCL=0
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'I2C : ACKNOWLEDGE (mis par le PicBasic)
    '===========================================================================
I2C_ACK:
    OUT SCL,0                                                 'SCL=0
    OUT SDA,0                                                 'SDA=0
    OUT SCL,1                                                 'SCL=1
    OUT SCL,0                                                 'SCL=0
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'I2C : NO ACKNOWLEDGE (mis par le PicBasic)
    '===========================================================================
I2C_NOACK:
    OUT SCL,0                                                 'SCL=0
    OUT SCL,1                                                 'SCL=1
    OUT SCL,0                                                 'SCL=0
    RETURN                                                    'Retour au programme d'appel

    '===========================================================================
    'CONVERSION NOMBRE CODE SUR 3 CHIFFRES ASCII -> DECIMAL
    '===========================================================================
ASCII2DEC:
    i=0                                                       '
    GET(j)=GET(j)-48                                          'ASCII->DEC
    i=GET(j)*100                                              '
    j=j+1                                                     '
    GET(j)=GET(j)-48                                          'ASCII->DEC
    i=i+GET(j)*10                                             '
    j=j+1                                                     '
    GET(j)=GET(j)-48                                          'ASCII->DEC
    i=i+GET(j)                                                '
    RETURN                                                    'Retour au programme d'appel


'*** FIN DU PROGRAMME **********************************************************